package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Business;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.interfaces.BusinessService;
import com.bocloud.sms.model.BusinessModel;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 * @time 2020/6/3 周三 15:29
 */
@RestController
@RequestMapping("/v1/bizs")
@Tag(name = "业务管理")
@Slf4j
public class BusinessController {

    private final BusinessService businessService;

    public BusinessController(BusinessService businessService) {
        this.businessService = businessService;
    }

    /**
     * 查看业务列表
     *
     * @param pager 分页参数
     * @return 分页查询数据
     */
    @Operation(summary = "查看业务列表")
    @GetMapping
    public GeneralResult<GridBean<Business>> list(Pager pager,
                                                  @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.list(pager, requestContext);
    }

    /**
     * 添加业务
     *
     * @param businessModel  要添加的业务
     * @param requestContext 请求上下文
     * @return 是否添加成功
     */
    @Operation(summary = "添加业务")
    @PostMapping
    public Result create(@RequestBody BusinessModel businessModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.create(businessModel, requestContext);
    }

    /**
     * 修改业务
     *
     * @param id             被修改的业务id
     * @param businessModel  修改的数据
     * @param requestContext 请求上下文
     * @return 修改结果
     */
    @Operation(summary = "修改业务")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody BusinessModel businessModel, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.modify(id, businessModel, requestContext);
    }

    /**
     * 删除业务
     *
     * @param id             被删除的业务id
     * @param requestContext 请求上下文
     * @return 删除结果
     */
    @Operation(summary = "删除业务")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.remove(id, requestContext);
    }

    /**
     * 批量删除业务
     *
     * @param ids            被删除的业务id列表
     * @param requestContext 请求上下文
     * @return 删除结果
     */
    @Operation(summary = "批量删除业务")
    @DeleteMapping
    public Result batchRemove(@RequestBody List<Long> ids,
                              @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.batchRemove(ids, requestContext);
    }

    /**
     * 查看业务明细
     *
     * @param id 被查看的业务id
     * @return 业务实体类
     */
    @Operation(summary = "查看业务明细")
    @GetMapping("/{id}")
    public GeneralResult<Business> detail(@PathVariable(value = Common.ID) Long id) {
        return businessService.detail(id);
    }

    /**
     * 查看业务下项目
     *
     * @param id 业务id
     * @return 项目列表
     */
    @Operation(summary = "查看业务下项目")
    @GetMapping("/{id}/projects")
    public GeneralResult<List<Project>> projects(
            @PathVariable(value = Common.ID) Long id) {
        return businessService.listBusinessProject(id);
    }

    /**
     * 配置业务项目
     *
     * @param id         业务id
     * @param projectIds 项目id列表
     * @return 配置结果
     */
    @Operation(summary = "配置业务项目")
    @PatchMapping("/{id}")
    public Result businessAction(@PathVariable(value = Common.ID) Long id,
                                 @RequestBody List<Long> projectIds) {
        return businessService.businessAction(id, projectIds);
    }

    /**
     * 导出业务列表
     *
     * @param pager          分页参数
     * @param requestContext 上下文
     */
    @Deprecated
    @Operation(summary = "导出项目列表")
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager,
                       @Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request,
                       HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = businessService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "业务统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            log.error("export project error,", e);
        }
    }

    /**
     * 下载业务列表模板
     *
     * @param response 浏览器响应
     */
    @Operation(summary = "下载业务列表模板")
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletResponse response) {
        businessService.importTemp(response);
    }

    /**
     * 导入业务列表
     *
     * @param requestContext 请求上下文
     * @param request        请求参数
     * @return 导入结果
     */
    @Operation(summary = "导入业务列表")
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return businessService.importData(multipartFile, requestContext);
    }

}

