package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.sms.model.LicenseMask;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.License;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

/**
 * 服务节点信息
 *
 * @author weiwei
 */
@Slf4j
@RestController
@RequestMapping("/v1")
@Tag(name = "服务节点信息管理")
public class ConfigController {

    private static final String LICENSE_SUCCESS = "证书激活成功";
    private static final String LICENSE_FAILED = "证书激活失败";
    private final LicenseDaemon licenseDaemon;
    @Value("${service.host:127.0.0.1}")
    private String host;
    @Value("${server.port:18088}")
    private String port;
    @Autowired
    private AmqpTemplate amqpTemplate;

    public ConfigController(CurrentService currentService) {
        this.licenseDaemon = LicenseDaemon.instance(currentService);
    }


    /**
     * 查询云管license信息
     *
     * @return license信息
     */
    @Operation(summary = "查询云管license信息")
    @GetMapping("/config/license")
    public GeneralResult<License> license() {
        License license = licenseDaemon.content();
        return new GeneralResult<>(true, license, "查询成功");
    }

    /**
     * 系统状态信息
     *
     * @param value 系统url
     * @return 系统状态信息
     */
    @Operation(summary = "系统状态信息")
    @GetMapping("/config/status")
    public GeneralResult status(@RequestParam(value = Common.VALUE) String value) {
        HttpClient client = new HttpClient();
        String message = client.get(value).getMessage();
        return JSONObject.parseObject(message, GeneralResult.class);
    }

    /**
     * 安装证书
     *
     * @param file 证书文件
     * @return 安装结果
     */
    @Operation(summary = "安装证书")
    @PostMapping("/config/license/install")
    public Result licenseInstall(@RequestParam("file") MultipartFile file) {
        if (file.isEmpty()) {
            return new Result(false, "证书为空!");
        }
        Result result;
        try {
            result = licenseDaemon.verify(file.getBytes());
            if (result.isFailed()) {
                log.error("证书校验失败");
                return new Result(false, LICENSE_FAILED);
            }
            result = licenseDaemon.install(file.getBytes());
            if (result.isSuccess()) {
                return new Result(true, LICENSE_SUCCESS);
            }
            return new Result(false, LICENSE_FAILED);
        } catch (IOException e) {
            log.error("证书激活异常：", e);
            return new Result(false, LICENSE_FAILED);
        }
    }

    /**
     * 安装license验证码
     *
     * @param licenseMask license验证码
     * @return 安装结果
     */
    @Operation(summary = "安装license验证码")
    @PostMapping("/config/captcha/install")
    public Result captchaInstall(@RequestBody LicenseMask licenseMask) {
        try {
            GeneralResult<byte[]> GeneralResult = licenseDaemon.decode(licenseMask.getLicenseMask());
            if (GeneralResult.isFailed()) {
                log.error("证书激活码格式有误，激活失败!");
                return new Result(false, LICENSE_FAILED);
            }
            Result result = licenseDaemon.verify(GeneralResult.getData());
            if (result.isFailed()) {
                log.error("证书格式不正确!");
                return new Result(false, LICENSE_FAILED);
            }
            result = licenseDaemon.install(GeneralResult.getData());
            if (result.isFailed()) {
                log.error("证书安装失败");
                return new Result(false, LICENSE_FAILED);
            }
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    "license.install.queue", LICENSE_SUCCESS);
            return new Result(true, LICENSE_SUCCESS);
        } catch (Exception e) {
            log.error("证书激活异常！", e);
            return new Result(false, LICENSE_FAILED);
        }
    }

    /**
     * 系统sid信息
     *
     * @return sid信息
     */
    @Operation(summary = "系统状态信息")
    @GetMapping("/status/sid")
    public GeneralResult getSid() {
        HttpClient client = new HttpClient();
        String url = "http://" + host + ":" + port + "/status/sid";
        String message = client.get(url).getMessage();
        return JSONObject.parseObject(message, GeneralResult.class);
    }

    /**
     * 系统sid信息
     *
     * @return sid信息列表
     */
    @Operation(summary = "系统状态信息")
    @GetMapping("/status/sids")
    public GeneralResult getSids() {
        HttpClient client = new HttpClient();
        String url = "http://" + host + ":" + port + "/status/sids";
        String message = client.get(url).getMessage();
        return JSONObject.parseObject(message, GeneralResult.class);
    }
}
