package com.bocloud.sms.service.upgrade;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

@Slf4j
public class UpgradeUtils {



    /**
     * 获取表的列名称列表
     *
     * @param dataSource 数据源
     * @param table      表名称
     * @return 列名称列表
     */
    public static List<String> getColumns(DataSource dataSource, String table) {
        NamedParameterJdbcTemplate jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        String sql = "select COLUMN_NAME from information_schema.COLUMNS " +
                "where TABLE_SCHEMA = database() and TABLE_NAME = :table";
        Map<String, Object> params = Collections.singletonMap("table", table);
        List<Map<String, Object>> columnResult = jdbcTemplate.queryForList(sql, params);
        return columnResult.stream().map(map -> map.get("COLUMN_NAME").toString()).collect(Collectors.toList());
    }

    /**
     * 检查一个表在两个数据库中的列是否一致，如果一致则返回列名称列表，否则抛异常
     *
     * @param dataSource1 第一个数据源
     * @param dataSource2 第二个数据源
     * @param table 表名
     * @return 返回列名称列表
     */
    public static List<String> checkColumns(DataSource dataSource1, DataSource dataSource2, String table) {
        List<String> columns1 = getColumns(dataSource1, table);
        List<String> columns2 = getColumns(dataSource2, table);
        // 判断列数量是否相等
        Assert.isTrue(columns1.size() == columns2.size(),
                String.format("Unequal column number. Table：%s, dataSource1: %d, dataSource2: %d",
                        table, columns1.size(), columns2.size()));
        // 判断列名称是否一致，记得要先排序
        columns1.sort(String::compareTo);
        columns2.sort(String::compareTo);
        Assert.isTrue(columns1.equals(columns2),
                String.format("Unequal column name. Table：%s, dataSource1: %s, dataSource2: %s",
                        table, String.join(",", columns1), String.join(",", columns2)));
        return columns2;
    }

    /**
     * 将列名称用``包裹起来并用,隔开
     * @param columns 列
     * @return 例如：`id`,`name`,`value`......
     */
    public static String getColumnNameSql(List<String> columns) {
        return columns.stream().map(name -> "`" + name + "`").collect(Collectors.joining(", "));
    }

    /**
     * 将列名称前加:并用,隔开
     * @param columns 列
     * @return 例如：:id,:name,:value......
     */
    public static String getColumnValueSql(List<String> columns) {
        return columns.stream().map(name -> ":" + name).collect(Collectors.joining(", "));
    }

    /**
     * 从源库中查询数据插入到新库中
     * 源库和新库中表字段名称必须完全一致，否则会抛异常
     *
     * @param fromDataSource 源数据源
     * @param targetDataSource 目标数据源
     * @param table 表名
     */
    public static void syncTable(DataSource fromDataSource, DataSource targetDataSource, String table) {
        syncTable(fromDataSource, targetDataSource, table, (Consumer<Integer>) null);
    }

    /**
     * 从源库中查询数据插入到新库中
     * 源库和新库中表字段名称必须完全一致，否则会抛异常
     *
     * @param fromDataSource 源数据源
     * @param targetDataSource 目标数据源
     * @param table 表名
     * @param callback 进度回调，取值范围0-100
     */
    public static void syncTable(DataSource fromDataSource, DataSource targetDataSource, String table, Consumer<Integer> callback) {
        // 先获取列名称列表
        List<String> columeNames = checkColumns(fromDataSource, targetDataSource, table);
        // 导数据
        syncTable(fromDataSource, targetDataSource, table, columeNames.toArray(new String[0]), callback);
    }

    /**
     * 从源库中查询数据插入到新库中，需要指定待处理的列
     *
     * @param fromDataSource 源数据源
     * @param targetDataSource 目标数据源
     * @param table 表名
     * @param columns 列名称
     */
    public static void syncTable(DataSource fromDataSource, DataSource targetDataSource, String table, String[] columns) {
        syncTable(fromDataSource, targetDataSource, table, columns, null);
    }

    /**
     * 从源库中查询数据插入到新库中，需要指定待处理的列
     *
     * @param fromDataSource 源数据源
     * @param targetDataSource 目标数据源
     * @param table 表名
     * @param columns 列名称
     * @param callback 进度回调，取值范围0-100
     */
    public static void syncTable(DataSource fromDataSource, DataSource targetDataSource, String table, String[] columns, Consumer<Integer> callback) {
        syncTable(fromDataSource, targetDataSource, table, table, columns, callback);
    }

    /**
     * 从源库中查询数据插入到新库中，需要指定待处理的列
     *
     * @param fromDataSource 源数据源
     * @param targetDataSource 目标数据源
     * @param fromTable 源表名
     * @param targetTable 目标表名
     * @param columns 列名称
     * @param callback 进度回调，取值范围0-100
     */
    public static void syncTable(DataSource fromDataSource, DataSource targetDataSource, String fromTable,String targetTable, String[] columns, Consumer<Integer> callback) {
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(fromDataSource);
        NamedParameterJdbcTemplate targetJdbcTemplate = new NamedParameterJdbcTemplate(targetDataSource);
        // 导数据
        long start = System.currentTimeMillis();
        String sqlCount = "select count(1) as count from " + fromTable + " where 1=1 ";
        List<Map<String, Object>> countResult = fromJdbcTemplate.queryForList(sqlCount, new HashMap<>());
        int count = Integer.parseInt(countResult.get(0).get("count").toString());
        log.debug("Count table: {}, lines: {}, uses: {}ms", fromTable, count, System.currentTimeMillis() - start);

        String sqlInsert = "insert into " + targetTable + "(" + getColumnNameSql(Arrays.asList(columns)) + ") " +
                "values (" + getColumnValueSql(Arrays.asList(columns)) + ") ";
        int current = 0, index = 1;
        start = System.currentTimeMillis();
        while (true) {
            String sqlSelect = "select " + getColumnNameSql(Arrays.asList(columns)) + " from " + fromTable + " where 1=1 " +
                    "limit " + (current * 100) + ", 100";
            List<Map<String, Object>> datas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
            if (datas.isEmpty()) {
                break;
            }
            for (Map<String, Object> data : datas) {
                targetJdbcTemplate.update(sqlInsert, data);
                if (callback != null) {
                    callback.accept(Math.floorDiv(index * 100, count));
                }
                index++;
            }
            current++;
        }
        log.debug("Sync table: {}, lines: {}, uses: {}ms/line", fromTable, count, System.currentTimeMillis() - start);
    }

    /**
     * 把某个表的某个字段从旧值更新到新值
     * update table set column = newValue where column = oldValue
     *
     * @param dataSource 数据源
     * @param table 表名称
     * @param column 字段名称
     * @param oldValue 旧值
     * @param newValue 新值
     */
    public static void updateColumn(DataSource dataSource, String table, String column, Object oldValue, Object newValue) {
        NamedParameterJdbcTemplate jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        String sqlUpdate = "update " + table + " set " + column + " = :new ";
        Map<String, Object> sqlParams = new HashMap<>();
        sqlParams.put("new", newValue);
        if (oldValue == null) {
            sqlUpdate += "where " + column + " is null ";
        } else {
            sqlUpdate += "where " + column + " = :old ";
            sqlParams.put("old", oldValue);
        }
        jdbcTemplate.update(sqlUpdate, sqlParams);
    }

    /**
     * 根据equalKey的值是否相同，获取idKey对应关系
     *
     * @param v50Datas 5.0信息
     * @param v55Datas 5.5信息
     * @param idKey id字段所在的key
     * @param equalKey 比较字段所在的key
     * @return 对应关系，key=5.0 value=5.5
     */
    private static Map<Long, Long> getIdMap(List<Map<String, Object>> v50Datas, List<Map<String, Object>> v55Datas, String idKey, String equalKey) {
        Map<Long, Long> idMap = new HashMap<>();
        for (Map<String, Object> oldUser : v50Datas) {
            Long id50 = Long.parseLong(oldUser.get(idKey).toString());
            String account50 = oldUser.get(equalKey).toString();
            for (Map<String, Object> newUser : v55Datas) {
                Long id55 = Long.parseLong(newUser.get(idKey).toString());
                String account55 = newUser.get(equalKey).toString();
                if (account50.equals(account55)) {
                    idMap.put(id50, id55);
                    break;
                }
            }
        }
        return idMap;
    }

    /**
     * (该方法为5.0-5.5升级特有方法，后续版本考虑删除)
     *
     * 获取5.0到5.5的用户id对应关系，用来更新creator_id, owner_id, mender_id。
     * 由于5.0中存在两个表，manager表和user表，这两个表会出现重复的id和重复的account
     * 并且我们无法区分creator_id, owner_id, mender_id对应的是manager表还是user表
     * 所以，如果某个id在manager表和user表同时都有，则认为来源于manager表
     *
     * @param dataSource50 5.0的数据源
     * @param dataSource55 5.5的数据源
     * @return 对应关系，key=5.0 value=5.5
     */
    public static Map<Long, Long> getIdMapOfManager(DataSource dataSource50, DataSource dataSource55) {
        NamedParameterJdbcTemplate v50JdbcTemplate = new NamedParameterJdbcTemplate(dataSource50);
        NamedParameterJdbcTemplate v55JdbcTemplate = new NamedParameterJdbcTemplate(dataSource55);
        // 查询5.0的用户
        String sql50Manager = "select id, account from manager where is_deleted = 0 ";
        String sql50User = "select id, account from users where is_deleted = 0 ";
        List<Map<String, Object>> v50ManagerResult = v50JdbcTemplate.queryForList(sql50Manager, new HashMap<>());
        List<Map<String, Object>> v50UserResult = v50JdbcTemplate.queryForList(sql50User, new HashMap<>());
        // 查询5.5的用户
        String sql55Manager = "select id, account from users where is_deleted = 0 and is_manager = 1";
        String sql55User = "select id, account from users where is_deleted = 0 and is_manager = 0";
        List<Map<String, Object>> v55ManagerResult = v55JdbcTemplate.queryForList(sql55Manager, new HashMap<>());
        List<Map<String, Object>> v55UserResult = v55JdbcTemplate.queryForList(sql55User, new HashMap<>());
        // 分别获取管理端和用户端的用户id对应关系
        Map<Long, Long> managerIdMap = getIdMap(v50ManagerResult, v55ManagerResult, "id", "account");
        Map<Long, Long> userIdMap = getIdMap(v50UserResult, v55UserResult, "id", "account");
        // 直接将managerIdMap和userIdMap合并，重合的id用managerIdMap覆盖userIdMap
        Map<Long, Long> finalIdMap = new HashMap<>(userIdMap);
        finalIdMap.putAll(managerIdMap);
        return finalIdMap;
    }

    public static Map<Long, Long> getIdMapOfUser(DataSource dataSource50, DataSource dataSource55) {
        NamedParameterJdbcTemplate v50JdbcTemplate = new NamedParameterJdbcTemplate(dataSource50);
        NamedParameterJdbcTemplate v55JdbcTemplate = new NamedParameterJdbcTemplate(dataSource55);
        // 查询5.0的用户
        String sql50User = "select id, account from users where is_deleted = 0 ";
        List<Map<String, Object>> v50UserResult = v50JdbcTemplate.queryForList(sql50User, new HashMap<>());
        // 查询5.5的用户
        String sql55User = "select id, account from users where is_deleted = 0 and is_manager = 0";
        List<Map<String, Object>> v55UserResult = v55JdbcTemplate.queryForList(sql55User, new HashMap<>());
        // 获取用户id对应关系
        return getIdMap(v50UserResult, v55UserResult, "id", "account");
    }

    /**
     * (该方法为5.0-5.5升级特有方法，后续版本考虑删除)
     *
     * 5.0中的CloudService表和5.5的CloudService表中的id有变化，所有要根据code进行比对，获取id的对应关系
     *
     * @param dataSource50 5.0的数据源
     * @param dataSource55 5.5的数据源
     * @return 对应关系，key=5.0 value=5.5
     */
    public static Map<Long, Long> getIdMapOfCloudService(DataSource dataSource50, DataSource dataSource55) {
        NamedParameterJdbcTemplate v50JdbcTemplate = new NamedParameterJdbcTemplate(dataSource50);
        NamedParameterJdbcTemplate v55JdbcTemplate = new NamedParameterJdbcTemplate(dataSource55);
        // 查询5.0的CloudService
        String sql = "select id, code from cloud_service where is_deleted = 0 ";
        List<Map<String, Object>> v50Result = v50JdbcTemplate.queryForList(sql, new HashMap<>());
        List<Map<String, Object>> v55Result = v55JdbcTemplate.queryForList(sql, new HashMap<>());
        return getIdMap(v50Result, v55Result, "id", "code");
    }

    /**
     * (该方法为5.0-5.5升级特有方法，后续版本考虑删除)
     *
     * 5.0中的ServiceCategory表和5.5的ServiceCategory表中的id有变化，所有要根据code和ServiceId进行比对，获取id的对应关系
     *
     * @param dataSource50 5.0的数据源
     * @param dataSource55 5.5的数据源
     * @return 对应关系，key=5.0 value=5.5
     */
    public static Map<Long, Long> getIdMapOfServiceCategory(DataSource dataSource50, DataSource dataSource55) {
        NamedParameterJdbcTemplate v50JdbcTemplate = new NamedParameterJdbcTemplate(dataSource50);
        NamedParameterJdbcTemplate v55JdbcTemplate = new NamedParameterJdbcTemplate(dataSource55);
        // 查询5.0的CloudService
        String sql = "select id, concat(service_id, '::', code) compare from service_category where is_deleted = 0 ";
        List<Map<String, Object>> v50Result = v50JdbcTemplate.queryForList(sql, new HashMap<>());
        List<Map<String, Object>> v55Result = v55JdbcTemplate.queryForList(sql, new HashMap<>());
        return getIdMap(v50Result, v55Result, "id", "compare");
    }

    /**
     * (该方法为5.0-5.5升级特有方法，后续版本考虑删除)
     *
     * 5.0中的ServiceSku表和5.5的ServiceSku表中的id有变化，所有要根据spec进行比对，获取id的对应关系
     *
     *
     * @param dataSource50 5.0的数据源
     * @param dataSource55 5.5的数据源
     * @return 对应关系，key=5.0 value=5.5
     */
    public static Map<Long, Long> getIdMapOfServiceSkuCompute(DataSource dataSource50, DataSource dataSource55) {
        Map<Long, Long> idMap = new HashMap<>();
        NamedParameterJdbcTemplate v50JdbcTemplate = new NamedParameterJdbcTemplate(dataSource50);
        NamedParameterJdbcTemplate v55JdbcTemplate = new NamedParameterJdbcTemplate(dataSource55);
        String sql = "select a.id, a.spec, a.service_code, b.code as catalog_code " +
                "from service_sku a inner join service_catalog b on a.catalog_id = b.id " +
                "where a.is_deleted = 0 ";
        List<Map<String, Object>> v50Result = v50JdbcTemplate.queryForList(sql, new HashMap<>());
        List<Map<String, Object>> v55Result = v55JdbcTemplate.queryForList(sql, new HashMap<>());
        for (Map<String, Object> v50Data : v50Result) {
            long v50Id = Long.parseLong(v50Data.get("id").toString());
            String v50ServiceCode = v50Data.get("service_code").toString();
            String v50CatalogCode = v50Data.get("catalog_code").toString();
            List<ServiceSkuSpec> v50Spec = Optional.of(v50Data).map(data -> data.get("spec")).map(Object::toString)
                    .filter(StringUtils::isNotBlank).map(data -> JSON.parseArray(data, ServiceSkuSpec.class))
                    .orElseGet(ArrayList::new);

            for (Map<String, Object> v55Data : v55Result) {
                long v55Id = Long.parseLong(v50Data.get("id").toString());
                String v55ServiceCode = v55Data.get("service_code").toString();
                String v55CatalogCode = v55Data.get("catalog_code").toString();
                List<ServiceSkuSpec> v55Spec = Optional.of(v55Data).map(data -> data.get("spec")).map(Object::toString)
                        .filter(StringUtils::isNotBlank).map(data -> JSON.parseArray(data, ServiceSkuSpec.class))
                        .orElseGet(ArrayList::new);

                if ("compute".equals(v50CatalogCode) && "compute".equals(v55CatalogCode) && v50ServiceCode.equals(v55ServiceCode)) {
                    // 计算类型，比对几核几G，如果一样则认为是同一个规格
                    // 5.0spec例如[{"unit":"核","specName":"cpu","specValue":"4"},
                    //            {"unit":"GB","specName":"memory","specValue":"4"}]
                    // 5.5spec例如[{"unit":"核","specName":"cpu","displayName":"CPU","specValue":"32"},
                    //            {"unit":"GB","specName":"memory","displayName":"内存","specValue":"2"}]
                    ServiceSkuSpec v50Cpu = v50Spec.stream().filter(spec -> "cpu".equals(spec.getSpecName())).findFirst().orElse(null);
                    ServiceSkuSpec v50Memory = v50Spec.stream().filter(spec -> "memory".equals(spec.getSpecName())).findFirst().orElse(null);
                    ServiceSkuSpec v55Cpu = v55Spec.stream().filter(spec -> "cpu".equals(spec.getSpecName())).findFirst().orElse(null);
                    ServiceSkuSpec v55Memory = v55Spec.stream().filter(spec -> "memory".equals(spec.getSpecName())).findFirst().orElse(null);
                    if (v50Cpu != null && v50Memory != null && v55Cpu != null && v55Memory != null
                            && Objects.equals(v50Cpu.getUnit(), v55Cpu.getUnit())
                            && Objects.equals(v50Cpu.getSpecValue(), v55Cpu.getSpecValue())) {
                        idMap.put(v50Id, v55Id);
                    }
                } else if ("storage".equals(v50CatalogCode) && "storage".equals(v55CatalogCode) && v50ServiceCode.equals(v55ServiceCode)) {

                }





            }
        }





        return idMap;
    }

    public static class ServiceSkuSpec {
        private String specName;
        private String displayName;
        private String specValue;
        private String unit;
        public String getSpecName() {
            return specName;
        }
        public void setSpecName(String specName) {
            this.specName = specName;
        }
        public String getDisplayName() {
            return displayName;
        }
        public void setDisplayName(String displayName) {
            this.displayName = displayName;
        }
        public String getSpecValue() {
            return specValue;
        }
        public void setSpecValue(String specValue) {
            this.specValue = specValue;
        }
        public String getUnit() {
            return unit;
        }
        public void setUnit(String unit) {
            this.unit = unit;
        }
    }

}
