package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.AccountSecurity;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.repository.AccountSecurityRepository;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.ldap.LdapConnector;
import com.bocloud.sms.service.ldap.LdapRepository;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.encrypt.SHAEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.util.*;

@Slf4j
public class LdapTransportor {

    private final UserRepository userRepository;
    private final AccountSecurityRepository accountSecurityRepository;
    private final DepartmentRepository departmentRepository;
    private LdapConnector ldapConnector;
    private LdapRepository ldapRepository;

    public LdapTransportor(UserRepository userRepository, DepartmentRepository departmentDao, AccountSecurityRepository accountSecurityRepository, LdapConfig config) {
        super();
        this.userRepository = userRepository;
        this.departmentRepository = departmentDao;
        this.accountSecurityRepository = accountSecurityRepository;
        init(config);
    }

    public boolean accessable() {
        return ldapConnector.accessable();
    }

    private void init(LdapConfig config) {
        this.ldapConnector = new LdapConnector(config.getUrl(), config.getAccount(), config.getPassword(),
                config.getBaseDn());
        this.ldapRepository = new LdapRepository(ldapConnector);
        ldapConnector.start();
    }

    /**
     * 同步
     *
     * @return
     */
    public GeneralResult sync() {
        try {
            if (!ldapConnector.accessable()) {
                return new GeneralResult(false, "Ldap平台认证失败");
            }
            List<Department> deptLists = listChild(0L);
            List<User> managerList = userRepository.listByOrigin(null, "LDAP", false);
            JSONObject object = ldapRepository.listAll("");
            List<Department> depts = JSONArray.parseArray(object.getString("depts"), Department.class);
            List<User> managers = JSONArray.parseArray(object.getString("managers"), User.class);
            log.info("start sync ldap data to db……");
            syncDept(deptLists, depts);
            syncUser(managerList, managers, null, null, false);
            log.info("end sync ldap data to db……");
        } catch (Exception e) {
            log.info("sync ldap info error:", e);
        } finally {
            if (null != ldapConnector) {
                ldapConnector.close();
            }
        }
        return new GeneralResult(true, "success");
    }

    /**
     * 同步组织机构
     *
     * @throws Exception
     */
    private void syncDept(List<Department> oldDepts, List<Department> newDepts) throws Exception {
        if (ListTool.isEmpty(newDepts)) {
            return;
        }
        Map<Long, Department> idMap = new HashMap<>(16, 0.75F);
        if (ListTool.isEmpty(oldDepts)) {
            syncAllToDb(newDepts, 0L, idMap);
            return;
        }
        Map<String, Department> deptLdapMap = deptMap(newDepts, "");
        syncDept(oldDepts, deptLdapMap, "", idMap);
        syncDeptToDb(newDepts, deptLdapMap, "", 0L, idMap);
    }

    /**
     * 同步所有数据到数据库
     *
     * @param departments
     * @param parentId
     * @param idMap
     * @throws Exception
     */
    private void syncAllToDb(List<Department> departments, Long parentId, Map<Long, Department> idMap)
            throws Exception {
        if (ListTool.isEmpty(departments)) {
            return;
        }
        for (Department department : departments) {
            List<Department> children = department.getChildren();
            List<User> managers = department.getUsers();
            department.setCreatorId(1L);
            department.setMenderId(1L);
            department.setTenantId(0L);
            department.setGmtCreate(new Date());
            department.setGmtModify(department.getGmtCreate());
            department.setStatus("NORMAL");
            department.setParentId(parentId);
            department.setOrigin("LDAP");
            departmentRepository.save(department);
            idMap.put(department.getId(), department);
            String ids = "[" + getDeptIds(department.getId() + "", department.getParentId(), idMap) + "]";
            saveUsers(managers, department.getId(), ids, true);
            syncAllToDb(children, department.getId(), idMap);
        }
    }

    private String getDeptIds(String ids, Long parentId, Map<Long, Department> idMap) {
        Department parent = idMap.get(parentId);
        if (null == parent) {
            return ids;
        }
        if (parent.getParentId() == 0) {
            return parentId + "," + ids;
        }
        ids = parentId + "," + ids;
        return getDeptIds(ids, parent.getParentId(), idMap);
    }

    private void saveUsers(List<User> managers, Long deptId, String deptIds, boolean hasDept) throws Exception {
        if (ListTool.isEmpty(managers)) {
            return;
        }
        List<List<User>> managerLists = new ArrayList<>();
        List<User> userList = new ArrayList<>();
        Map<String, String> saltMap = new HashMap<>(16, 0.75F);
        Encryptor encryptor = new SHAEncryptor();
        for (User manager : managers) {
            manager.setCreatorId(1L);
            manager.setMenderId(1L);
            manager.setGmtCreate(new Date());
            manager.setGmtModify(manager.getGmtCreate());
            manager.setStatus("NORMAL");
            manager.setDepartId(deptId);
            manager.setOrigin("LDAP");
            manager.setIsManager(true);
            String salt = UUID.randomUUID().toString();
            String password = !StringUtils.hasText(manager.getPassword()) ? "123456" : manager.getPassword();
            manager.setPassword(encryptor.encrypt(password, salt));
            userList.add(manager);
            saltMap.put(manager.getAccount(), salt);
            if (userList.size() == 1000) {
                managerLists.add(userList);
                userList = new ArrayList<>();
            }
        }
        if (userList.size() > 0) {
            managerLists.add(userList);
        }
        for (List<User> list : managerLists) {
            if (list.size() > 0) {
                userRepository.batchSave(list, list.size());
            }
        }
        List<User> allUsers = this.userRepository.listByOrigin(deptId, "LDAP", hasDept);
        for (User manager : allUsers) {
            if (!StringUtils.hasText(manager.getAccount()) || !saltMap.containsKey(manager.getAccount())) {
                continue;
            }
            AccountSecurity security = new AccountSecurity();
            String data = Long.toString(System.currentTimeMillis());
            security.setTargetId(manager.getId());
            security.setCategory(AccountSecurity.Category.User);
            security.setTenantId(0L);
            security.setSalt(saltMap.get(manager.getAccount()));
            security.setApiKey(encryptor.encrypt(data, Common.API_KEY));
            security.setSecKey(encryptor.encrypt(data, Common.SEC_KEY));
            accountSecurityRepository.save(security);
        }
    }

    /**
     * 更新数据库已有的组织机构数据
     *
     * @param departments
     * @param deptLdapMap
     * @param name
     * @throws Exception
     */
    private void syncDept(List<Department> departments, Map<String, Department> deptLdapMap, String name,
                          Map<Long, Department> idMap) throws Exception {
        for (Department department : departments) {
            idMap.put(department.getId(), department);
            String dn = "ou=" + department.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            if (deptLdapMap.containsKey(dn)) {
                Department dept = deptLdapMap.get(dn);
                String ids = "[" + getDeptIds(department.getId() + "", department.getParentId(), idMap) + "]";
                syncUser(department.getUsers(), dept.getUsers(), department.getId(), ids, true);
                deptLdapMap.remove(dn);
                if (!ListTool.isEmpty(department.getChildren())) {
                    syncDept(department.getChildren(), deptLdapMap, dn, idMap);
                }
            }
        }
    }

    /**
     * 同步用户
     *
     * @param dbUsers
     * @param ldapUsers
     * @param deptId
     * @throws Exception
     */
    private void syncUser(List<User> dbUsers, List<User> ldapUsers, Long deptId, String deptIds, boolean hasDept)
            throws Exception {
        if (ListTool.isEmpty(ldapUsers)) {
            return;
        }
        if (ListTool.isEmpty(dbUsers)) {
            saveUsers(ldapUsers, deptId, deptIds, hasDept);
            return;
        }
        Map<String, User> deptUserMap = new HashMap<>(16, 0.75F);
        for (User manager : ldapUsers) {
            deptUserMap.put(manager.getAccount(), manager);
        }
        Encryptor encryptor = new SHAEncryptor();
        for (User manager : dbUsers) {
            if (deptUserMap.containsKey(manager.getAccount())) {
                User ldapUser = deptUserMap.get(manager.getAccount());
                if (!StringUtils.hasText(manager.getAccount()) || !manager.getAccount().equals(ldapUser.getAccount())) {
                    manager.setAccount(ldapUser.getAccount());
                    manager.setMenderId(1L);
                    manager.setGmtModify(new Date());
                    userRepository.update(manager);
                }
                deptUserMap.remove(manager.getAccount());
            }
        }
        for (String key : deptUserMap.keySet()) {
            User manager = deptUserMap.get(key);
            manager.setCreatorId(1L);
            manager.setMenderId(1L);
            manager.setGmtCreate(new Date());
            manager.setGmtModify(manager.getGmtCreate());
            manager.setStatus("NORMAL");
            manager.setDepartId(deptId);
            manager.setOrigin("LDAP");
            manager.setIsManager(true);
            String salt = UUID.randomUUID().toString();
            String password = !StringUtils.hasText(manager.getPassword()) ? "123456" : manager.getPassword();
            manager.setPassword(encryptor.encrypt(password, salt));
            userRepository.save(manager);
            AccountSecurity security = new AccountSecurity();
            String data = Long.toString(System.currentTimeMillis());
            security.setTargetId(manager.getId());
            security.setCategory(AccountSecurity.Category.User);
            security.setTenantId(0L);
            security.setSalt(salt);
            security.setApiKey(encryptor.encrypt(data, Common.API_KEY));
            security.setSecKey(encryptor.encrypt(data, Common.SEC_KEY));
            accountSecurityRepository.save(security);
        }
    }

    /**
     * 同步组织机构到数据库
     *
     * @param departments
     * @param deptLdapMap
     * @param name
     * @param parentId
     * @throws Exception
     */
    private void syncDeptToDb(List<Department> departments, Map<String, Department> deptLdapMap, String name,
                              Long parentId, Map<Long, Department> idMap) throws Exception {
        if (ListTool.isEmpty(departments)) {
            return;
        }
        for (Department department : departments) {
            String dn = "ou=" + department.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            if (!deptLdapMap.containsKey(dn)) {
                continue;
            }
            List<User> managers = department.getUsers();
            department.setCreatorId(1L);
            department.setMenderId(1L);
            department.setTenantId(0L);
            department.setGmtCreate(new Date());
            department.setGmtModify(department.getGmtCreate());
            department.setStatus("NORMAL");
            department.setParentId(parentId);
            department.setOrigin("LDAP");
            departmentRepository.save(department);
            idMap.put(department.getId(), department);
            String ids = "[" + getDeptIds(department.getId() + "", department.getParentId(), idMap) + "]";
            saveUsers(managers, department.getId(), ids, true);
            syncDeptToDb(department.getChildren(), deptLdapMap, dn, department.getId(), idMap);
        }
    }

    private List<Department> listChild(Long parentId) throws Exception {
        List<Department> departs = departmentRepository.list(parentId, 0L, "LDAP");
        if (!ListTool.isEmpty(departs)) {
            for (Department depart : departs) {
                depart.setChildren(listChild(depart.getId()));
                depart.setUsers(userRepository.listByOrigin(depart.getId(), "LDAP", true));
            }
        }
        return departs;
    }

    private Map<String, Department> deptMap(List<Department> departs, String name) {
        if (ListTool.isEmpty(departs)) {
            return new HashMap<>(1);
        }
        Map<String, Department> deptMap = new HashMap<>(16, 0.75F);
        for (Department depart : departs) {
            String dn = "ou=" + depart.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            deptMap.put(dn, depart);
            deptMap.putAll(deptMap(depart.getChildren(), dn));
        }
        return deptMap;
    }

}
